@@ -19,6 +19,7 @@ class Experiment:
     def __init__(self,
         environments    : Sequence[Environment],
         learners        : Sequence[Learner],
+        description     : str = None,
         learner_task    : LearnerTask     = SimpleLearnerTask(),
         environment_task: EnvironmentTask = SimpleEnvironmentTask(),
         evaluation_task : EvaluationTask  = OnlineOnPolicyEvalTask()) -> None:
@@ -27,6 +28,7 @@ class Experiment:
         Args:
             environments: The collection of environments to use in the experiment.
             learners: The collection of learners to use in the experiment.
+            description: A description of the experiment for documentaiton purposes.
             learner_task: A task which describes a learner.
             environment_task: A task which describes an environment.
             evaluation_task: A task which evaluates a learner on an environment.
@@ -34,6 +36,7 @@ class Experiment:
 
         self._environments     = environments
         self._learners         = learners
+        self._description      = description
         self._learner_task     = learner_task
         self._environment_task = environment_task
         self._evaluation_task  = evaluation_task
@@ -97,8 +100,8 @@ class Experiment:
         """The maximum number of tasks allowed in a chunk before breaking a chunk into smaller chunks."""
         return self._maxtasksperchunk if self._maxtasksperchunk is not None else CobaContext.experiment.maxtasksperchunk
 
-    def evaluate(self, result_file:str = None) -> Result:
-        """Evaluate the experiment and return the results.
+    def run(self, result_file:str = None) -> Result:
+        """Run the experiment and return the results.
 
         Args:
             result_file: The file for writing and restoring results .
@@ -110,14 +113,14 @@ class Experiment:
         else:
             CobaContext.logger = DecoratedLogger([ExceptLog()], CobaContext.logger, [StampLog()])
 
-        restored = Result.from_file(result_file) if result_file and Path(result_file).exists() else Result()
+        restored = Result.from_file(result_file) if result_file and Path(result_file).exists() else None
 
         n_given_learners     = len(self._learners)
         n_given_environments = len(self._environments)
 
-        if len(restored.experiment) != 0:
-            assert n_given_learners     == restored.experiment['n_learners'    ], "The current experiment doesn't match the given transaction log."
-            assert n_given_environments == restored.experiment['n_environments'], "The current experiment doesn't match the given transaction log."
+        if restored:
+            assert n_given_learners     == restored.experiment.get('n_learners',n_given_learners)        , "The current experiment doesn't match the given transaction log."
+            assert n_given_environments == restored.experiment.get('n_environments',n_given_environments), "The current experiment doesn't match the given transaction log."
 
         workitems  = CreateWorkItems(self._environments, self._learners, self._learner_task, self._environment_task, self._evaluation_task)
         unfinished = RemoveFinished(restored)
@@ -130,9 +133,8 @@ class Experiment:
         process        = multi_process if mp > 1 or mc != 0 else single_process
 
         try:
-            if not restored: sink.write(["T0", n_given_learners, n_given_environments])
+            if not restored: sink.write(["T0", {'n_learners':n_given_learners, 'n_environments':n_given_environments, 'description':self._description }])
             Pipes.join(workitems, unfinished, process, Foreach(sink)).run()
-
         except KeyboardInterrupt as e: # pragma: no cover
             CobaContext.logger.log("Experiment execution was manually aborted via Ctrl-C")
 
@@ -143,3 +145,12 @@ class Experiment:
             CobaContext.logger = CobaContext.logger.undecorate()
 
         return sink.read()
+
+    def evaluate(self, result_file:str = None) -> Result:
+        """Evaluate the experiment and return the results (this is a backwards compatible proxy for the run method).
+
+        Args:
+            result_file: The file for writing and restoring results .
+        """
+
+        return self.run(result_file=result_file)
